#include "../../include/freenet/identityxml.h"

#ifdef XMEM
	#include <xmem.h>
#endif

IdentityXML::IdentityXML()
{
	Initialize();
}

std::string IdentityXML::GetXML()
{
	TiXmlDocument td;
	TiXmlDeclaration *tdec=new TiXmlDeclaration("1.0","UTF-8","");
	TiXmlElement *tid;
	TiXmlPrinter tp;

	td.LinkEndChild(tdec);
	tid=new TiXmlElement("Identity");
	td.LinkEndChild(tid);

	tid->LinkEndChild(XMLCreateCDATAElement("Name",m_name));

	tid->LinkEndChild(XMLCreateBooleanElement("SingleUse",m_singleuse));

	tid->LinkEndChild(XMLCreateBooleanElement("PublishTrustList",m_publishtrustlist));

	tid->LinkEndChild(XMLCreateBooleanElement("PublishBoardList",m_publishboardlist));

	// freesite edition will be -1 if identity isn't publishing a freesite
	if(m_freesiteedition>=0)
	{
		tid->LinkEndChild(XMLCreateTextElement("FreesiteEdition",m_freesiteedition));
	}

	td.Accept(&tp);
	return std::string(tp.CStr());

}

void IdentityXML::Initialize()
{
	m_name="";
	m_publishtrustlist=false;
	m_publishboardlist=false;
	m_singleuse=false;
	m_freesiteedition=-1;
}

const bool IdentityXML::ParseXML(const std::string &xml)
{
	TiXmlDocument td;
	td.Parse(xml.c_str());

	if(!td.Error())
	{
		TiXmlText *txt;
		TiXmlHandle hnd(&td);

		Initialize();

		txt=hnd.FirstChild("Identity").FirstChild("Name").FirstChild().ToText();
		if(txt)
		{
			m_name=txt->ValueStr();
		}

		if(m_name.size()>40)
		{
			m_name.erase(40);
		}

		m_singleuse=XMLGetBooleanElement(hnd.FirstChild("Identity").ToElement(),"SingleUse");

		m_publishtrustlist=XMLGetBooleanElement(hnd.FirstChild("Identity").ToElement(),"PublishTrustList");

		m_publishboardlist=XMLGetBooleanElement(hnd.FirstChild("Identity").ToElement(),"PublishBoardList");

		txt=hnd.FirstChild("Identity").FirstChild("FreesiteEdition").FirstChild().ToText();
		if(txt)
		{
			std::string editionstr=SanitizeSingleString(txt->ValueStr());
			StringFunctions::Convert(editionstr,m_freesiteedition);
		}

		return true;

	}
	else
	{
		return false;
	}

}
